/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package romancalculator.implementation;

import romancalculator.framework.DataInput;
import romancalculator.framework.DataOutput;
import romancalculator.framework.EquationBuilder;
import romancalculator.framework.EquationNode;
import romancalculator.framework.InvalidInputException;
import romancalculator.framework.Lexer;
import romancalculator.framework.NoInputException;
import romancalculator.framework.RomanCalculator;
import romancalculator.framework.RomanTranslator;
import romancalculator.framework.TokenStream;

public class RomanCalculatorImpl implements RomanCalculator {

	private Lexer lexer;
	private RomanTranslator translator;
	private EquationBuilder builder;

	public RomanCalculatorImpl(Lexer lexer, RomanTranslator translator, EquationBuilder builder) {
		this.lexer = lexer;
		this.translator = translator;
		this.builder = builder;
	}

	@Override
	public void solve(DataInput in, DataOutput out) {
		String rawInput;
		try {
			rawInput = in.getInput();
		} catch (NoInputException e) {
			out.setOutput("error: No Input");
			return;
		}
		TokenStream tokens = lexer.tokenize(rawInput);
		TokenStream translatedTokens = new RomanTranslatingTokenStream(tokens, translator);
		EquationNode equationRoot;
		try {
			equationRoot = builder.buildEquationTree(translatedTokens);
		} catch (InvalidInputException e) {
			out.setOutput("error : Invalid Input");
			return;
		}
		int result = equationRoot.evaluate();
		String romanResult = translator.toRoman(result);
		out.setOutput(romanResult);
	}

}
