/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package exercise;

import exercise.stub.Database;
import exercise.stub.DatabaseException;
import exercise.stub.Employee;
import exercise.stub.EmployeeManager;
import static java.util.Arrays.asList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import static org.hamcrest.CoreMatchers.allOf;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;
import org.junit.Test;
import static org.junit.internal.matchers.IsCollectionContaining.hasItem;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class EmployeeManagerTest {

	@Test
	public void testHappy() throws DatabaseException {
		Database data = mock(Database.class);
		Map<String, String> employee1 = new HashMap<>();
		employee1.put("name", "John");
		employee1.put("surname", "Doe");
		employee1.put("salary", "100000");
		Map<String, String> employee2 = new HashMap<>();
		employee2.put("name", "John");
		employee2.put("surname", "Black");
		employee2.put("salary", "80000");
		when(data.query(anyString())).thenReturn(asList(employee1, employee2));
		EmployeeManager manager = new EmployeeManager(data);
		Collection<Employee> selected = manager.getByName("John");
		assertThat(selected, allOf(
				hasItem(new Employee("John", "Doe", 100000)),
				hasItem(new Employee("John", "Black", 80000))
		));
	}

	@Test
	public void testException() throws DatabaseException {
		Database data = mock(Database.class);
		when(data.query(anyString())).thenThrow(new DatabaseException());
		EmployeeManager manager = new EmployeeManager(data);
		Collection<Employee> selected = manager.getByName("John");
		assertThat(selected.size(), is(0));
	}

	@Test
	public void testInvalidData() throws DatabaseException {
		Database data = mock(Database.class);
		Map<String, String> employee1 = new HashMap<>();
		employee1.put("name", "John");
		employee1.put("surname", "Doe");
		employee1.put("salary", "A lot");
		Map<String, String> employee2 = new HashMap<>();
		employee2.put("name", "John");
		employee2.put("surname", "Black");
		employee2.put("salary", "80000");
		when(data.query(anyString())).thenReturn(asList(employee1, employee2));
		EmployeeManager manager = new EmployeeManager(data);
		Collection<Employee> selected = manager.getByName("John");
		assertThat(selected, allOf(
				hasItem(new Employee("John", "Black", 80000))
		));
	}

}
